package extensions;

import java.io.IOException;

import java.net.Socket;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import dareka.common.Logger;
import dareka.extensions.Extension;
import dareka.extensions.RequestFilter;
import dareka.extensions.ExtensionManager;
import dareka.processor.HttpHeader;
import dareka.processor.HttpRequestHeader;
import dareka.processor.Resource;
import dareka.processor.StringResource;
import dareka.processor.Processor;
import dareka.processor.impl.Cache;

/*
 *
 */
public class liveInfoExtension implements Extension, Processor, RequestFilter {

	// Extension interface
	public Object queryInterface(Extension.Type type) {
		if (type == Extension.Type.Extension1)
			return (Extension) this;
		if (type == Extension.Type.Processor1)
			return (Processor) this;
		if (type == Extension.Type.RequestFilter1)
			return (RequestFilter) this;
		return null;
	}

	public void registerExtensions(ExtensionManager mgr) {
		mgr.registerProcessor(this);
		mgr.registerRequestFilter(this);
	}

	public String getVersionString() {
		return "liveInfoExtension (9).01+20121106_2mod";
	}

	public liveInfoExtension() throws IOException {
	}

	private static final String[] PROCESSOR_SUPPORTED_METHODS =
		new String[] { "GET" };

	public String[] getSupportedMethods() {
		return PROCESSOR_SUPPORTED_METHODS;
	}

	private static final Pattern PROCESSOR_SUPPORTED_PATTERN = Pattern.compile(
			"^http://live\\.nicovideo\\.jp/cache/liveinfo");

	private static final Pattern LIVE_CKEY_PATTERN = Pattern.compile(
			"^http://.*\\.live\\.nicovideo\\.jp/api/getckey.*[&?]id=(\\w{2}\\d+)");

	private static final Pattern LIVE_ID_PATTERN = Pattern.compile(
			"^http://.*\\.live\\.nicovideo\\.jp/api/getckey.*[&?]referer_id=(lv\\d+)");

	public Pattern getSupportedURLAsPattern() {
		return PROCESSOR_SUPPORTED_PATTERN;
	}

	public String getSupportedURLAsString() {
		return null;
	}

	private static String lvid = "";
	private static String smid = "";
	private static String idIsCache = "none";

	public int onRequest(HttpRequestHeader requestHeader) throws IOException {
		String uri = requestHeader.getURI();
		Matcher m1 = LIVE_ID_PATTERN.matcher(uri);
		Matcher m = LIVE_CKEY_PATTERN.matcher(uri);
		if (m1.find() && m.find()) {
			lvid = m1.group(1);
			smid = m.group(1);
			String isCache = Cache.getType(smid);
			if (isCache != null) {
				if (isCache.equals("")) {
					idIsCache = "cache";
				} else {
					idIsCache = "low";
				}
			} else {
				idIsCache = "none";
			}
		}
		return RequestFilter.OK;
	}

	public Resource onRequest(HttpRequestHeader requestHeader, Socket browser)
			throws IOException {
		String str = lvid+","+smid+","+idIsCache;
		StringResource r = new StringResource(str);
		r.addNoCacheResponseHeaders();
		r.addResponseHeader(HttpHeader.CONTENT_TYPE, "text/plain");
		return r;
	}

}


